@@ -1,4 +1,4 @@
-//@line 39 "/builds/tinderbox/Fx-Trunk/Linux_2.6.18-53.1.13.el5_Depend/mozilla/browser/components/nsBrowserGlue.js"
+//@line 39 "/builds/tinderbox/Fx-Mozilla1.9-Release/Linux_2.6.18-53.1.13.el5_Depend/mozilla/browser/components/nsBrowserGlue.js"
 
 const Ci = Components.interfaces;
 const Cc = Components.classes;
@@ -187,6 +187,10 @@ BrowserGlue.prototype = {
 
     // handle any UI migration
     this._migrateUI();
+
+    const osvr = Cc['@mozilla.org/observer-service;1'].
+                 getService(Ci.nsIObserverService);
+    osvr.notifyObservers(null, "browser-ui-startup-complete", "");
   },
 
   // profile shutdown handler (contains profile cleanup routines)
@@ -527,13 +531,29 @@ BrowserGlue.prototype = {
       this._dataSource = this._rdf.GetDataSource("rdf:local-store");
       this._dirty = false;
 
-      var currentSet = this._rdf.GetResource("currentset");
-
-      // get an nsIRDFResource for the nav-bar item
-      var navBar = this._rdf.GetResource("chrome://browser/content/browser.xul#nav-bar");
-      var target = this._getPersist(navBar, currentSet);
-      if (target && !/(?:^|,)unified-back-forward-button(?:$|,)/.test(target))
-        this._setPersist(navBar, currentSet, "unified-back-forward-button," + target);
+      let currentsetResource = this._rdf.GetResource("currentset");
+      let toolbars = ["nav-bar", "toolbar-menubar", "PersonalToolbar"];
+      for (let i = 0; i < toolbars.length; i++) {
+        let toolbar = this._rdf.GetResource("chrome://browser/content/browser.xul#" + toolbars[i]);
+        let currentset = this._getPersist(toolbar, currentsetResource);
+        if (!currentset) {
+          // toolbar isn't customized
+          if (i == 0)
+            // new button is in the defaultset, nothing to migrate
+            break;
+          continue;
+        }
+        if (/(?:^|,)unified-back-forward-button(?:$|,)/.test(currentset))
+          // new button is already there, nothing to migrate
+          break;
+        if (/(?:^|,)back-button(?:$|,)/.test(currentset)) {
+          let newset = currentset.replace(/(^|,)back-button($|,)/,
+                                          "$1unified-back-forward-button,back-button$2")
+          this._setPersist(toolbar, currentsetResource, newset);
+          // done migrating
+          break;
+        }
+      }
 
       // force the RDF to be saved
       if (this._dirty)
@@ -744,3 +764,4 @@ BrowserGlue.prototype = {
 function NSGetModule(aCompMgr, aFileSpec) {
   return XPCOMUtils.generateModule([BrowserGlue]);
 }
+
